#include "stm32f10x.h"
#include "sys.c"
#include "io.c"
#include "i3202lcd.c"
#include "tsc2003.c"
#include "ASCII2.fnt"
#include "hg2.fnt"
#include "graphic.c"
#include "drawtext.c"


void mystrcpy(unsigned char const *src, unsigned char *dst)
{
  unsigned char n;
  for(n=0;src[n]!='\0';n++) dst[n]=src[n];
  dst[n]='\0';
}

void lcd_clear(void)
{                  
  unsigned int n,m;
  
  LcdSetWindow(0,0,320,240);
  LcdDataStart();
  for(n=0;n<320;n++)
   {  for(m=0;m<240;m++) { LcdDataWrite(0xffff); }
   }
  LcdDataEnd();
}


unsigned char mystrsize(unsigned char *str)
{
  unsigned char n;
  for(n=0;str[n]!='\0';n++) {}
  return n;
}

void rect(unsigned int x, unsigned int y, unsigned int width, unsigned int height, unsigned int rgb)
{   unsigned int n;

    LcdSetWindow(x,y,width,1);
    LcdDataStart();
    for(n=0;n<width;n++) LcdDataWrite(rgb);
    LcdDataEnd();
    
    LcdSetWindow(x,y,1,height);
    LcdDataStart();
    for(n=0;n<height;n++) LcdDataWrite(rgb);
    LcdDataEnd();    
    
    LcdSetWindow(x+width,y,1,height);
    LcdDataStart();
    for(n=0;n<height;n++) LcdDataWrite(rgb);
    LcdDataEnd();        
    
    LcdSetWindow(x,y+height,width,1);
    LcdDataStart();
    for(n=0;n<width;n++) LcdDataWrite(rgb);
    LcdDataEnd();
}   

#define NUMBER_OF_ICON 14

#define ICON_CLEAR   0
#define ICON_PEN1    1
#define ICON_PEN2    2
#define ICON_PEN3    3
#define ICON_ERS1    4
#define ICON_ERS2    5
#define ICON_C_R     6
#define ICON_C_G     7
#define ICON_C_B     8
#define ICON_C_RG    9
#define ICON_C_RB    10
#define ICON_C_BG    11
#define ICON_C_BK    12
#define ICON_C_WH    13




typedef struct {
unsigned int x;
unsigned int y;
unsigned int width;
unsigned int height;
unsigned int rgb;
unsigned char name[10];
} MENU;

MENU menu[NUMBER_OF_ICON];



void make_icon(void)
{   unsigned char icon;
    unsigned char n,m;  
    unsigned int text_x,text_y;
    
    for(icon=0;icon<NUMBER_OF_ICON;icon++)
     {
        LcdSetWindow( menu[icon].x , menu[icon].y , menu[icon].width , menu[icon].height);
        LcdDataStart();
        for(n=0;n<menu[icon].height;n++)
          {  for(m=0;m<menu[icon].width;m++)
               {  LcdDataWrite(menu[icon].rgb);
               }
          }
        LcdDataEnd();
                         
        text_x=mystrsize(&menu[icon].name[0])*8;       
        text_y=16;                                     
        if(text_x>menu[icon].width) text_x=0;          
        else {  text_x=(menu[icon].width-text_x)/2;  } 
        if(text_y>menu[icon].height) text_y=0;         
        else {  text_y=(menu[icon].height-text_y)/2;  }
        LcdAsciiStringCT(menu[icon].x+text_x , menu[icon].y+text_y , &menu[icon].name[0], 0x0000);
        
        rect( menu[icon].x , menu[icon].y , menu[icon].width , menu[icon].height , 0x0000);  // outline
     }  
}

#define ICONSIZE_X  33 
#define ICONSIZE_Y  33
void make_menu(void)
{
   menu[ICON_CLEAR].x=320-ICONSIZE_X;    menu[ICON_CLEAR].y=0;            menu[ICON_CLEAR].width=ICONSIZE_X; menu[ICON_CLEAR].height=ICONSIZE_Y; menu[ICON_CLEAR].rgb=0xffff;  mystrcpy("CLR",&menu[ICON_CLEAR].name[0]);
   menu[ICON_PEN1].x=320-ICONSIZE_X;     menu[ICON_PEN1].y=ICONSIZE_Y;    menu[ICON_PEN1].width=ICONSIZE_X;  menu[ICON_PEN1].height=ICONSIZE_Y;  menu[ICON_PEN1].rgb=0xffff;   mystrcpy("P1",&menu[ICON_PEN1].name[0]);  
   menu[ICON_PEN2].x=320-ICONSIZE_X;     menu[ICON_PEN2].y=ICONSIZE_Y*2;  menu[ICON_PEN2].width=ICONSIZE_X;  menu[ICON_PEN2].height=ICONSIZE_Y;  menu[ICON_PEN2].rgb=0xffff;   mystrcpy("P2",&menu[ICON_PEN2].name[0]);  
   menu[ICON_PEN3].x=320-ICONSIZE_X;     menu[ICON_PEN3].y=ICONSIZE_Y*3;  menu[ICON_PEN3].width=ICONSIZE_X;  menu[ICON_PEN3].height=ICONSIZE_Y;  menu[ICON_PEN3].rgb=0xffff;   mystrcpy("P3",&menu[ICON_PEN3].name[0]);  
   menu[ICON_ERS1].x=320-ICONSIZE_X;     menu[ICON_ERS1].y=ICONSIZE_Y*4;  menu[ICON_ERS1].width=ICONSIZE_X;  menu[ICON_ERS1].height=ICONSIZE_Y;  menu[ICON_ERS1].rgb=0xffff;   mystrcpy("E1",&menu[ICON_ERS1].name[0]);  
   menu[ICON_ERS2].x=320-ICONSIZE_X;     menu[ICON_ERS2].y=ICONSIZE_Y*5;  menu[ICON_ERS2].width=ICONSIZE_X;  menu[ICON_ERS2].height=ICONSIZE_Y;  menu[ICON_ERS2].rgb=0xffff;   mystrcpy("E2",&menu[ICON_ERS2].name[0]);  

   menu[ICON_C_R].x=0;                   menu[ICON_C_R].y=240-ICONSIZE_Y; menu[ICON_C_R].width=ICONSIZE_X;  menu[ICON_C_R].height=ICONSIZE_Y;  menu[ICON_C_R].rgb=RGB(0xff,0,0);   mystrcpy("",&menu[ICON_C_R].name[0]); 
   menu[ICON_C_G].x=ICONSIZE_X;          menu[ICON_C_G].y=240-ICONSIZE_Y; menu[ICON_C_G].width=ICONSIZE_X;  menu[ICON_C_G].height=ICONSIZE_Y;  menu[ICON_C_G].rgb=RGB(0,0xff,0);   mystrcpy("",&menu[ICON_C_G].name[0]); 
   menu[ICON_C_B].x=ICONSIZE_X*2;        menu[ICON_C_B].y=240-ICONSIZE_Y; menu[ICON_C_B].width=ICONSIZE_X;  menu[ICON_C_B].height=ICONSIZE_Y;  menu[ICON_C_B].rgb=RGB(0,0,0xff);   mystrcpy("",&menu[ICON_C_B].name[0]); 
   menu[ICON_C_RG].x=ICONSIZE_X*3;       menu[ICON_C_RG].y=240-ICONSIZE_Y; menu[ICON_C_RG].width=ICONSIZE_X;  menu[ICON_C_RG].height=ICONSIZE_Y;  menu[ICON_C_RG].rgb=RGB(0xff,0xff,0);    mystrcpy("",&menu[ICON_C_RG].name[0]); 
   menu[ICON_C_RB].x=ICONSIZE_X*4;       menu[ICON_C_RB].y=240-ICONSIZE_Y; menu[ICON_C_RB].width=ICONSIZE_X;  menu[ICON_C_RB].height=ICONSIZE_Y;  menu[ICON_C_RB].rgb=RGB(0xff,0,0xff);    mystrcpy("",&menu[ICON_C_RB].name[0]); 
   menu[ICON_C_BG].x=ICONSIZE_X*5;       menu[ICON_C_BG].y=240-ICONSIZE_Y; menu[ICON_C_BG].width=ICONSIZE_X;  menu[ICON_C_BG].height=ICONSIZE_Y;  menu[ICON_C_BG].rgb=RGB(0,0xff,0xff);    mystrcpy("",&menu[ICON_C_BG].name[0]); 
   menu[ICON_C_BK].x=ICONSIZE_X*6;       menu[ICON_C_BK].y=240-ICONSIZE_Y; menu[ICON_C_BK].width=ICONSIZE_X;  menu[ICON_C_BK].height=ICONSIZE_Y;  menu[ICON_C_BK].rgb=RGB(0,0,0);          mystrcpy("",&menu[ICON_C_BK].name[0]); 
   menu[ICON_C_WH].x=ICONSIZE_X*7;       menu[ICON_C_WH].y=240-ICONSIZE_Y; menu[ICON_C_WH].width=ICONSIZE_X;  menu[ICON_C_WH].height=ICONSIZE_Y;  menu[ICON_C_WH].rgb=RGB(0xff,0xff,0xff);   mystrcpy("",&menu[ICON_C_WH].name[0]); 
   
   make_icon();
}         

unsigned char menu_process(unsigned int x, unsigned int y)
{
   unsigned char icon;
   
   for(icon=0;icon<NUMBER_OF_ICON;icon++)
     {
        if((x>=menu[icon].x)&&(x<=(menu[icon].x+menu[icon].width)))
          {
             if((y>=menu[icon].y)&&(y<=(menu[icon].y+menu[icon].height)))  return icon;
          }
     }
   
   return 0xff;  // no selected..
}


void pain_pixel(unsigned int x, unsigned int y, unsigned int pixel_size, unsigned int rgb)
{  unsigned int n,m;

   m=pixel_size*pixel_size;
   LcdSetWindow(x,y,pixel_size,pixel_size);
   LcdDataStart();
   for(n=0;n<m;n++) LcdDataWrite(rgb);
   LcdDataEnd();
}

int main(void)
{

  unsigned int x,y; 
  unsigned char result;
  unsigned int color=0x0000;
  unsigned char tool=1;  
  
  gpio_init();
  rcc_init();  
  uart_init1();

  mdelay(1000);  // wait for flash loader communication...
  put_string1("\n Paint.");

  LcdInit(); 
  LcdBl_H;  // LCD Back Light On..
  
  
  LcdSetDirection(90);
  lcd_clear();
  
  Tsc2003Init();

  make_menu();
    
  while(1)
  {
     if(PenIrq) 
      { 
         result=average_touch_pos(&x,&y);  
         if(result)
          { 
            result=menu_process(x,y);
            switch(result)
             {
                case ICON_CLEAR: 
                  lcd_clear(); make_menu(); 
                break;
                case ICON_PEN1:
                  tool=1;
                break;
                case ICON_PEN2:
                  tool=2;
                break;
                case ICON_PEN3:
                  tool=3;
                break; 
                case ICON_ERS1:
                  tool=4;
                break;
                case ICON_ERS2: 
                  tool=5;
                break;
                
                case ICON_C_R:
                  color=RGB(0xff,0,0);
                break;
                case ICON_C_G:
                  color=RGB(0,0xff,0);
                break;
                case ICON_C_B:
                  color=RGB(0,0,0xff);
                break;
                case ICON_C_RG:
                  color=RGB(0xff,0xff,0);
                break;
                case ICON_C_RB:
                  color=RGB(0xff,0,0xff);
                break;
                case ICON_C_BG:
                  color=RGB(0,0xff,0xff);
                break;
                case ICON_C_BK:
                  color=RGB(0,0,0);
                break;
                case ICON_C_WH:
                  color=RGB(0xff,0xff,0xff);
                break;
                
                default:
                  if(tool==1)
                    {  pain_pixel(x,y,2,color);
                    }
                  else if(tool==2)
                    {  pain_pixel(x,y,4,color);
                    }
                  else if(tool==3)
                    {  pain_pixel(x,y,6,color);
                    }
                  else if(tool==4)
                    {  pain_pixel(x,y,9,0xffff);
                    }                    
                  else if(tool==5)
                    {  pain_pixel(x,y,15,0xffff);
                    }                                 
                break;             
             }
          } 
      }
     else
      {  window_cnt=0;   // reset touch window buffer.
      }
     
  }                                  
}
