// TSC2003 Touch IC Library by NKJ 2010.02.
                    
#define tsc_clock(X)    ((BBIO(GPIOB->ODR,6))=(X))

void tsc_data_wr(unsigned char x)
{
  GPIOB->CRL = (GPIOB->CRL&0x0fffffff)|0x30000000;  // GPIOB.7 Output Push-pull
  BBIO(GPIOB->ODR,7)=x;
}
unsigned char tsc_data_rd(void)
{
  GPIOB->CRL = (GPIOB->CRL&0x0fffffff)|0x40000000;  // GPIOB.7 Input Floating
  return BBIO(GPIOB->IDR,7);
}

void tsc_delay(void)
{
  volatile unsigned int n;  // about 7uS.
  for(n=0;n<60;n++);
}

void tsc_init(void)
{
  tsc_data_wr(1);
  tsc_clock(1);
}

void tsc_start(void)  // Start of transmission
{
  tsc_data_wr(0);
  tsc_delay();	
  tsc_clock(0);
}

void tsc_stop(void)   // Stop of transmission
{
  tsc_delay();
  tsc_data_wr(0);	
  tsc_clock(1);
  tsc_delay();	
  tsc_data_wr(1);
}            


unsigned char tsc_write(unsigned char ddata)
{ 
  unsigned char n;
  unsigned char ack;
   
  for(n=0;n<8;n++) 
   {
     tsc_delay();	   

     if( (ddata<<n)&0x80 ) tsc_data_wr(1); 
     else tsc_data_wr(0);  

     tsc_clock(1);
     tsc_delay();
     tsc_clock(0);
   }
   
  tsc_data_rd();  // read mode
  tsc_delay();
  ack=tsc_data_rd();  // for 9th bit.  
  tsc_clock(1);
  tsc_delay();
  tsc_clock(0);
  
  return ack;
}


unsigned char tsc_read(unsigned char ack)
{ 
  unsigned char n;
  unsigned char ddata=0;

  for(n=9;n>1;n--)
   {
     tsc_delay();
     tsc_clock(1);
     tsc_delay();

     ddata |= tsc_data_rd() << (n-2);

     tsc_clock(0);
   }

  tsc_delay();
  tsc_data_wr(ack);   // for read state	 
  tsc_clock(1);
  tsc_delay();
  tsc_clock(0);
   
  return ddata;
}
          

void tsc_send_cmd(unsigned char cmd)
{
   tsc_delay();

   tsc_start();  
   tsc_write(0x90);  // 1 0 0 1 0 0 0 0  write
   tsc_write(cmd);
   tsc_stop();  
}

unsigned int tsc_read_dat_12bit(void)
{  
   unsigned char ddata1,ddata2;
   unsigned int ddata=0;
               
   tsc_delay();

   tsc_start();  
   tsc_write(0x91);  // 1 0 0 1 0 0 0 1  read
   ddata1=tsc_read(0);
   ddata2=tsc_read(1);
   tsc_stop(); 
   
   ddata=(((unsigned int)ddata1<<8)&0xff00)|(ddata2&0xff);
   return ddata;
}                   

unsigned char tsc_read_dat_8bit(void)
{  
   unsigned char ddata;
               
   tsc_delay();

   tsc_start();  
   tsc_write(0x91);  // 1 0 0 1 0 0 0 1  read
   ddata=tsc_read(1);
   tsc_stop(); 
   
   return ddata;
}                   



#define TSC8BIT  0x02
void Tsc2003Init(void)
{
   tsc_init();
   tsc_send_cmd(0xc8|TSC8BIT);  //  Measure X Position , PENIRQ : enable , ADC : 12bit
}

unsigned char TscXPos(void)
{
   tsc_send_cmd(0xc8|TSC8BIT);
   return tsc_read_dat_8bit();
}

unsigned char TscYPos(void)
{
   tsc_send_cmd(0xd8|TSC8BIT);
   return tsc_read_dat_8bit();
}

unsigned char TscZ1Pos(void)
{
   tsc_send_cmd(0xe8|TSC8BIT);
   return tsc_read_dat_8bit();
}
                           
unsigned char TscZ2Pos(void)
{
   tsc_send_cmd(0xf8|TSC8BIT);
   return tsc_read_dat_8bit();
}


///////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////
//       calculate touch data                                //
//
//   DIF_VALUE  WINDOW_SIZE Ͽ ġ  
                 
#define X_ADC_START   0x0a           //
#define Y_ADC_START   0x07           //  
#define X_ADC_END     0xea           //
#define Y_ADC_END     0xef           //  ADC RAW VALUE

#define X_ADC_SIZE    (X_ADC_END-X_ADC_START)
#define Y_ADC_SIZE    (Y_ADC_END-Y_ADC_START)

#define X_TOLERANCE   12     //
#define Y_TOLERANCE   12     //  X , Y ġ 

#define X_LCD_SIZE    240
#define Y_LCD_SIZE   (320 + 25)   // 25 -> lcd's icon_bar


void get_touch_pos(unsigned int *x,unsigned int *y)
{  unsigned long adc_x, adc_y;
   unsigned long lcd_x, lcd_y;
   
   adc_x=TscXPos();
   lcd_x=(X_LCD_SIZE*adc_x)/X_ADC_SIZE;
   adc_y=TscYPos();
   lcd_y=(Y_LCD_SIZE*adc_y)/Y_ADC_SIZE;   
   
//   *x=lcd_x-X_TOLERANCE;              //
//   *y=Y_LCD_SIZE-lcd_y+Y_TOLERANCE;   // lcd 0 degree
   
   *x=Y_LCD_SIZE-lcd_y+Y_TOLERANCE;   // 
   *y=X_LCD_SIZE-lcd_x+X_TOLERANCE;   //  lcd 90 degree
}
      
#define DIF_VALUE   90       // +-DIF_VALUE ̻ ̳   return 0
unsigned char compare_xy(unsigned int x1,unsigned y1, unsigned int x2, unsigned int y2)
{  signed int difx,dify;

   difx=x2-x1;
   dify=y2-y1;
   
   if( (difx<=(-DIF_VALUE)) || (difx>=DIF_VALUE) ) return 0;  // 
   if( (dify<=(-DIF_VALUE)) || (dify>=DIF_VALUE) ) return 0;  // 
   
   return 1;
}


#define LIMIT_X       320
//#define LIMIT_X       (320+25) // 25 -> lcd's icon_bar
#define LIMIT_Y       240
#define WINDOW_SIZE   20
unsigned int xwindow[WINDOW_SIZE];
unsigned int ywindow[WINDOW_SIZE];
unsigned char window_cnt=0;
unsigned char average_touch_pos(unsigned int *x, unsigned int *y)
{                
   unsigned int nx,ny;
   unsigned char i;  
   unsigned int sumx=0,sumy=0;
   static unsigned int oldx,oldy;
   static unsigned char first_data=1;
   
   get_touch_pos(&nx,&ny);
   if( (nx>=LIMIT_X) || (ny>=LIMIT_Y) )  return 0;  // out of position 

   xwindow[window_cnt]=nx;
   ywindow[window_cnt]=ny;
   
   if(window_cnt==(WINDOW_SIZE-1))
    {
      if(first_data) 
        { first_data=0;
        }
      else 
        {   if(compare_xy(nx,ny,oldx,oldy)==0)  return 0;  // Ƣ touch data skip
        }
      for(i=0;i<WINDOW_SIZE;i++)
         {  sumx += xwindow[i];
            sumy += ywindow[i];
         }           
       for(i=0;i<(WINDOW_SIZE-1);i++)
         {  xwindow[i]=xwindow[i+1];   //  shift x data
            ywindow[i]=ywindow[i+1];   //  shift y data
         }  
       
       *x=sumx/WINDOW_SIZE;  
       *y=sumy/WINDOW_SIZE;
       oldx=*x; oldy=*y;
       return 1;   //  return x,y data
    }
   else
    {  window_cnt++;  
       first_data=1;
    }  
    
   return 0;  // not ready touch data
}


